import 'package:nexbounce/nexbounce.dart';

/// The subscription class.
///
/// It has the properties `Store store` and `Subscriber subscriber`.
///
/// Also has a `cancel` function to cancel subscription.
class Subscription {
  final Store store;
  final void Function() subscriber;

  const Subscription({required this.store, required this.subscriber});

  void cancel() => store._subscribers.remove(subscriber);
}

/// A base class that has to be extended to create stores.
abstract class Store {
  final _subscribers = <void Function()>{};
  final _debouncer = Nexbounce();

  /// Calls all subscribers.
  void setState(final void Function() callback) {
    callback();
    _publish();
  }

  /// Accepts a callback as its argument which is run
  /// after `setState` is called.
  ///
  /// Note that running callbacks is debounced.
  Subscription subscribe(final void Function() subscriber) {
    _subscribers.add(subscriber);
    return Subscription(store: this, subscriber: subscriber);
  }

  /// Same as `subscribe` but callback is immediately called for the first time.
  Subscription runAndSubscribe(final void Function() subscriber) {
    subscriber();
    return subscribe(subscriber);
  }

  void _publish() {
    _debouncer.enqueue(() {
      for (final subscriber in _subscribers) {
        subscriber();
      }
    });
  }
}
